@@ -31,14 +31,9 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static char sccsid[] = "@(#)telnet.c	8.4 (Berkeley) 5/30/95";
-#endif /* not lint */
-
-#include <config.h>
+/* based on @(#)telnet.c	8.1 (Berkeley) 6/6/93 */
 
 #include <sys/types.h>
-#include <stdlib.h>
 
 #if	defined(unix)
 #include <signal.h>
@@ -60,7 +55,7 @@ static char sccsid[] = "@(#)telnet.c	8.4 (Berkeley) 5/30/95";
 #include "general.h"
 
 
-#define	strip(x) ((my_want_state_is_wont(TELOPT_BINARY)) ? ((x)&0x7f) : (x))
+#define	strip(x)	((x)&0x7f)
 
 static unsigned char	subbuffer[SUBBUFSIZE],
 			*subpointer, *subend;	 /* buffer for sub-options */
@@ -97,6 +92,7 @@ int
 	askedSGA = 0,	/* We have talked about suppress go ahead */
 #endif	/* defined(TN3270) */
 	telnetport,
+        wantencryption = 0,
 	SYNCHing,	/* we are in TELNET SYNCH mode */
 	flushout,	/* flush output */
 	autoflush = 0,	/* flush output when interrupting? */
@@ -111,6 +107,8 @@ int
 
 char *prompt = 0;
 
+int scheduler_lockout_tty = 0;
+
 cc_t escape;
 cc_t rlogin;
 #ifdef	KLUDGELINEMODE
@@ -180,7 +178,7 @@ init_telnet()
     ClearArray(options);
 
     connected = In3270 = ISend = localflow = donebinarytoggle = 0;
-#if	defined(AUTHENTICATION) || defined(ENCRYPTION)
+#if	defined(AUTHENTICATION) || defined(ENCRYPTION) 
     auth_encrypt_connect(connected);
 #endif	/* defined(AUTHENTICATION) || defined(ENCRYPTION)  */
     restartany = -1;
@@ -372,9 +370,9 @@ willoption(option)
 #if	defined(AUTHENTICATION)
 	    case TELOPT_AUTHENTICATION:
 #endif
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 	    case TELOPT_ENCRYPT:
-#endif
+#endif /* ENCRYPTION */
 		new_state_ok = 1;
 		break;
 
@@ -404,10 +402,10 @@ willoption(option)
 	    }
 	}
 	set_my_state_do(option);
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 	if (option == TELOPT_ENCRYPT)
 		encrypt_send_support();
-#endif
+#endif	/* ENCRYPTION */
 }
 
 	void
@@ -495,9 +493,9 @@ dooption(option)
 	    case TELOPT_LFLOW:		/* local flow control */
 	    case TELOPT_TTYPE:		/* terminal type option */
 	    case TELOPT_SGA:		/* no big deal */
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 	    case TELOPT_ENCRYPT:	/* encryption variable option */
-#endif
+#endif	/* ENCRYPTION */
 		new_state_ok = 1;
 		break;
 
@@ -523,7 +521,8 @@ dooption(option)
 #endif
 
 	    case TELOPT_XDISPLOC:	/* X Display location */
-		if (env_getvalue((unsigned char *)"DISPLAY"))
+		if (env_getvalue((unsigned char *)"DISPLAY") &&
+		    env_is_exported((unsigned char *)"DISPLAY"))
 		    new_state_ok = 1;
 		break;
 
@@ -626,7 +625,7 @@ mklist(buf, name)
 	register char c, *cp, **argvp, *cp2, **argv, **avt;
 
 	if (name) {
-		if ((int)strlen(name) > 40) {
+		if (strlen(name) > 40) {
 			name = 0;
 			unknown[0] = name_unknown;
 		} else {
@@ -692,7 +691,7 @@ mklist(buf, name)
 		else if (islower(c))
 			*cp = toupper(c);
 	}
-
+	
 	/*
 	 * Check for an old V6 2 character name.  If the second
 	 * name points to the beginning of the buffer, and is
@@ -743,11 +742,11 @@ is_unique(name, as, ae)
 	return (1);
 }
 
-#ifdef	TERMCAP
+#ifndef	HAVE_SETUPTERM
 char termbuf[1024];
 
-#ifndef HAVE_SETUPTERM
-int
+	/*ARGSUSED*/
+	int
 setupterm(tname, fd, errp)
 	char *tname;
 	int fd, *errp;
@@ -762,7 +761,6 @@ setupterm(tname, fd, errp)
 		*errp = 0;
 	return(-1);
 }
-#endif
 #else
 #define	termbuf	ttytype
 extern char ttytype[];
@@ -786,7 +784,7 @@ gettermname()
 				(setupterm(tname, 1, &err) == 0)) {
 			tnamep = mklist(termbuf, tname);
 		} else {
-			if (tname && ((int)strlen(tname) <= 40)) {
+			if (tname && (strlen(tname) <= 40)) {
 				unknown[0] = tname;
 				upcase(tname);
 			} else
@@ -957,7 +955,8 @@ suboption()
 	    unsigned char temp[50], *dp;
 	    int len;
 
-	    if ((dp = env_getvalue((unsigned char *)"DISPLAY")) == NULL) {
+	    if (((dp = env_getvalue((unsigned char *)"DISPLAY")) == NULL) ||
+		(! env_is_exported((unsigned char *)"DISPLAY"))) {
 		/*
 		 * Something happened, we no longer have a DISPLAY
 		 * variable.  So, turn off the option.
@@ -1008,7 +1007,7 @@ suboption()
 	}
 	break;
 #endif
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 	case TELOPT_ENCRYPT:
 		if (SB_EOF())
 			return;
@@ -1068,7 +1067,7 @@ suboption()
 			break;
 		}
 		break;
-#endif
+#endif	/* ENCRYPTION */
     default:
 	break;
     }
@@ -1750,10 +1749,10 @@ telrcv()
 	}
 
 	c = *sbp++ & 0xff, scc--; count++;
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 	if (decrypt_input)
 		c = (*decrypt_input)(c);
-#endif
+#endif	/* ENCRYPTION */
 
 	switch (telrcv_state) {
 
@@ -1778,10 +1777,10 @@ telrcv()
 		*Ifrontp++ = c;
 		while (scc > 0) {
 		    c = *sbp++ & 0377, scc--; count++;
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 		    if (decrypt_input)
 			c = (*decrypt_input)(c);
-#endif
+#endif	/* ENCRYPTION */
 		    if (c == IAC) {
 			telrcv_state = TS_IAC;
 			break;
@@ -1800,10 +1799,10 @@ telrcv()
 	    if ((c == '\r') && my_want_state_is_dont(TELOPT_BINARY)) {
 		if (scc > 0) {
 		    c = *sbp&0xff;
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 		    if (decrypt_input)
 			c = (*decrypt_input)(c);
-#endif
+#endif	/* ENCRYPTION */
 		    if (c == 0) {
 			sbp++, scc--; count++;
 			/* a "true" CR */
@@ -1813,10 +1812,10 @@ telrcv()
 			sbp++, scc--; count++;
 			TTYADD('\n');
 		    } else {
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 		        if (decrypt_input)
 			    (*decrypt_input)(-1);
-#endif
+#endif	/* ENCRYPTION */
 
 			TTYADD('\r');
 			if (crmod) {
@@ -1838,7 +1837,7 @@ telrcv()
 	case TS_IAC:
 process_iac:
 	    switch (c) {
-
+	    
 	    case WILL:
 		telrcv_state = TS_WILL;
 		continue;
@@ -2209,6 +2208,10 @@ Scheduler(block)
     }
 #endif	/* defined(TN3270) && defined(unix) */
 
+    if (scheduler_lockout_tty) {
+	ttyin = ttyout = 0;
+    }
+
     /* Call to system code to process rings */
 
     returnValue = process_rings(netin, netout, netex, ttyin, ttyout, !block);
@@ -2224,7 +2227,7 @@ Scheduler(block)
 					ring_full_consecutive(&ttyiring));
 	    if (c) {
 		returnValue = 1;
-		ring_consumed(&ttyiring, c);
+	        ring_consumed(&ttyiring, c);
 	    }
 	} else {
 #   endif /* defined(TN3270) */
@@ -2251,9 +2254,11 @@ Scheduler(block)
 telnet(user)
     char *user;
 {
+    int printed_encrypt = 0;
+    
     sys_telnet_init();
 
-#if	defined(AUTHENTICATION) || defined(ENCRYPTION)
+#if	defined(AUTHENTICATION) || defined(ENCRYPTION) 
     {
 	static char local_host[256] = { 0 };
 
@@ -2264,17 +2269,19 @@ telnet(user)
 	auth_encrypt_init(local_host, hostname, "TELNET", 0);
 	auth_encrypt_user(user);
     }
-#endif
+#endif	/* defined(AUTHENTICATION) || defined(ENCRYPTION)  */
 #   if !defined(TN3270)
-    if (telnetport) {
 #if	defined(AUTHENTICATION)
-	if (autologin)
-		send_will(TELOPT_AUTHENTICATION, 1);
+    if (autologin)
+	send_will(TELOPT_AUTHENTICATION, 1);
 #endif
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
+    if (telnetport || wantencryption) {
 	send_do(TELOPT_ENCRYPT, 1);
 	send_will(TELOPT_ENCRYPT, 1);
-#endif
+    }
+#endif	/* ENCRYPTION */
+    if (telnetport) {
 	send_do(TELOPT_SGA, 1);
 	send_will(TELOPT_TTYPE, 1);
 	send_will(TELOPT_NAWS, 1);
@@ -2283,13 +2290,73 @@ telnet(user)
 	send_will(TELOPT_LINEMODE, 1);
 	send_will(TELOPT_NEW_ENVIRON, 1);
 	send_do(TELOPT_STATUS, 1);
-	if (env_getvalue((unsigned char *)"DISPLAY"))
+	if (env_getvalue((unsigned char *)"DISPLAY") &&
+	    env_is_exported((unsigned char *)"DISPLAY"))
 	    send_will(TELOPT_XDISPLOC, 1);
 	if (eight)
 	    tel_enter_binary(eight);
     }
 #   endif /* !defined(TN3270) */
 
+#ifdef ENCRYPTION
+    /*
+     * Note: we assume a tie to the authentication option here.  This
+     * is necessary so that authentication fails, we don't spin
+     * forever. 
+     */
+    if (wantencryption) {
+	extern int auth_has_failed;
+	time_t timeout = time(0) + 60;
+
+	send_do(TELOPT_ENCRYPT, 1);
+	send_will(TELOPT_ENCRYPT, 1);
+	while (1) {
+	    if (my_want_state_is_wont(TELOPT_AUTHENTICATION)) {
+		printf("\nServer refused to negotiation authentication, which is required\n");
+		printf("for encryption.  Good bye.\n\r");
+		Exit(1);
+	    }
+	    if (auth_has_failed) {
+		printf("\nAuthentication negotation has failed, which is required for\n");
+		printf("encryption.  Good bye.\n\r");
+		Exit(1);
+	    }
+	    if (my_want_state_is_dont(TELOPT_ENCRYPT) ||
+		my_want_state_is_wont(TELOPT_ENCRYPT)) {
+		printf("\nServer refused to negotiate encryption.  Good bye.\n\r");
+		Exit(1);
+	    }
+	    if (encrypt_is_encrypting())
+		break;
+	    if (time(0) > timeout) {
+		printf("\nEncryption could not be enabled.  Goodbye.\n\r");
+		Exit(1);
+	    }
+	    if (printed_encrypt == 0) {
+		    printed_encrypt = 1;
+		    printf("Waiting for encryption to be negotiated...");
+		    /*
+		     * Turn on MODE_TRAPSIG and then turn off localchars 
+		     * so that ^C will cause telnet to exit.
+		     */
+		    TerminalNewMode(getconnmode()|MODE_TRAPSIG);
+		    intr_waiting = 1;
+	    }
+	    if (intr_happened) {
+		    printf("\nUser requested an interrupt.  Goodbye.\n\r");
+		    Exit(1);
+	    }
+	    telnet_spin();
+	}
+	if (printed_encrypt) {
+		printf("done.\n");
+		intr_waiting = 0;
+		setconnmode(0);
+	}
+    }
+#endif
+
+
 #   if !defined(TN3270)
     for (;;) {
 	int schedValue;
@@ -2435,7 +2502,7 @@ netclear()
 		next = nextitem(next);
 	    } while (wewant(next) && (nfrontp > next));
 	    length = next-thisitem;
-	    memmove(good, thisitem, length);
+	    memcpy(good, thisitem, length);
 	    good += length;
 	    thisitem = next;
 	} else {
