@@ -31,11 +31,7 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static char sccsid[] = "@(#)state.c	8.5 (Berkeley) 5/30/95";
-#endif /* not lint */
-
-#include <config.h>
+/* based on @(#)state.c	8.1 (Berkeley) 6/4/93 */
 
 #include "telnetd.h"
 #if	defined(AUTHENTICATION)
@@ -48,11 +44,13 @@ unsigned char	will[] = { IAC, WILL, '%', 'c', 0 };
 unsigned char	wont[] = { IAC, WONT, '%', 'c', 0 };
 int	not42 = 1;
 
+static int envvarok P((char *));
+
 /*
  * Buffer for sub-options, and macros
  * for suboptions buffer manipulations
  */
-unsigned char subbuffer[512], *subpointer= subbuffer, *subend= subbuffer;
+unsigned char subbuffer[4096], *subpointer= subbuffer, *subend= subbuffer;
 
 #define	SB_CLEAR()	subpointer = subbuffer
 #define	SB_TERM()	{ subend = subpointer; SB_CLEAR(); }
@@ -79,9 +77,20 @@ unsigned char *subsave;
 #define	TS_SB		3	/* throw away begin's... */
 #define	TS_SE		4	/* ...end's (suboption negotiation) */
 #define	TS_WILL		5	/* will option negotiation */
-#define	TS_WONT		6	/* wont -''- */
-#define	TS_DO		7	/* do -''- */
-#define	TS_DONT		8	/* dont -''- */
+#define	TS_WONT		6	/* wont " */
+#define	TS_DO		7	/* do " */
+#define	TS_DONT		8	/* dont " */
+
+static void sb_auth_complete()
+{
+  if (!auth_negotiated) {
+    static char *error =
+      "An environment option was sent before authentication negotiation completed.\r\nThis may create a security hazard. Connection dropped.\r\n";
+    writenet(error, strlen(error));
+    netflush();
+    exit(1);
+  }
+}
 
 	void
 telrcv()
@@ -96,10 +105,10 @@ telrcv()
 		if ((&ptyobuf[BUFSIZ] - pfrontp) < 2)
 			break;
 		c = *netip++ & 0377, ncc--;
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 		if (decrypt_input)
 			c = (*decrypt_input)(c);
-#endif
+#endif	/* ENCRYPTION */
 		switch (state) {
 
 		case TS_CR:
@@ -128,10 +137,10 @@ telrcv()
 			 */
 			if ((c == '\r') && his_state_is_wont(TELOPT_BINARY)) {
 				int nc = *netip;
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 				if (decrypt_input)
 					nc = (*decrypt_input)(nc & 0xff);
-#endif
+#endif	/* ENCRYPTION */
 #ifdef	LINEMODE
 				/*
 				 * If we are operating in linemode,
@@ -144,10 +153,10 @@ telrcv()
 				} else
 #endif
 				{
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 					if (decrypt_input)
 						(void)(*decrypt_input)(-1);
-#endif
+#endif	/* ENCRYPTION */
 					state = TS_CR;
 				}
 			}
@@ -357,7 +366,7 @@ gotiac:			switch (c) {
 			continue;
 
 		default:
-			syslog(LOG_ERR, "telnetd: panic state=%d\n", state);
+			syslog(LOG_ERR, "telnetd: panic state=%d", state);
 			printf("telnetd: panic state=%d\n", state);
 			exit(1);
 		}
@@ -368,7 +377,7 @@ gotiac:			switch (c) {
 		char	xbuf2[BUFSIZ];
 		register char *cp;
 		int n = pfrontp - opfrontp, oc;
-		memmove(xptyobuf, opfrontp, n);
+		memcpy(xptyobuf, opfrontp, n);
 		pfrontp = opfrontp;
 		pfrontp += term_input(xptyobuf, pfrontp, n, BUFSIZ+NETSLOP,
 					xbuf2, &oc, BUFSIZ);
@@ -390,7 +399,7 @@ gotiac:			switch (c) {
  * All state defaults are negative, and resp defaults to 0.
  *
  * When initiating a request to change state to new_state:
- *
+ * 
  * if ((want_resp == 0 && new_state == my_state) || want_state == new_state) {
  *	do nothing;
  * } else {
@@ -468,7 +477,7 @@ extern void doclientstat();
 #endif
 #ifdef	ENCRYPTION
 extern void encrypt_send_support();
-#endif
+#endif	/* ENCRYPTION */
 
 	void
 willoption(option)
@@ -587,8 +596,8 @@ willoption(option)
 			func = encrypt_send_support;
 			changeok++;
 			break;
-#endif
-			
+#endif	/* ENCRYPTION */
+
 		default:
 			break;
 		}
@@ -651,8 +660,7 @@ willoption(option)
 		case TELOPT_ENCRYPT:
 			func = encrypt_send_support;
 			break;
-#endif
-
+#endif	/* ENCRYPTION */
 		case TELOPT_LFLOW:
 			func = flowstat;
 			break;
@@ -719,6 +727,7 @@ wontoption(option)
 			 */
 			if (lmodetype != REAL_LINEMODE)
 				break;
+			lmodetype = KLUDGE_LINEMODE;
 # endif	/* KLUDGELINEMODE */
 			clientstat(TELOPT_LINEMODE, WONT, 0);
 			break;
@@ -942,11 +951,11 @@ dooption(option)
 			/* NOT REACHED */
 			break;
 
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 		case TELOPT_ENCRYPT:
 			changeok++;
 			break;
-#endif
+#endif	/* ENCRYPTION */
 		case TELOPT_LINEMODE:
 		case TELOPT_TTYPE:
 		case TELOPT_NAWS:
@@ -1110,6 +1119,8 @@ suboption()
 	if (his_state_is_wont(TELOPT_TSPEED))	/* Ignore if option disabled */
 		break;
 
+	sb_auth_complete();
+	
 	settimer(tspeedsubopt);
 
 	if (SB_EOF() || SB_GET() != TELQUAL_IS)
@@ -1129,30 +1140,29 @@ suboption()
     }  /* end of case TELOPT_TSPEED */
 
     case TELOPT_TTYPE: {		/* Yaaaay! */
-	static char terminalname[41];
+	char *tt;
 
 	if (his_state_is_wont(TELOPT_TTYPE))	/* Ignore if option disabled */
 		break;
+	sb_auth_complete();
 	settimer(ttypesubopt);
 
 	if (SB_EOF() || SB_GET() != TELQUAL_IS) {
 	    return;		/* ??? XXX but, this is the most robust */
 	}
 
-	terminaltype = terminalname;
+	tt = terminaltype;
 
-	while ((terminaltype < (terminalname + sizeof terminalname-1)) &&
-								    !SB_EOF()) {
+	while ((tt < (terminaltype + sizeof(terminaltype) - 1)) && !SB_EOF()) {
 	    register int c;
 
 	    c = SB_GET();
 	    if (isupper(c)) {
 		c = tolower(c);
 	    }
-	    *terminaltype++ = c;    /* accumulate name */
+	    *tt++ = c;    /* accumulate name */
 	}
-	*terminaltype = 0;
-	terminaltype = terminalname;
+	*tt = 0;
 	break;
     }  /* end of case TELOPT_TTYPE */
 
@@ -1252,6 +1262,7 @@ suboption()
     case TELOPT_XDISPLOC: {
 	if (SB_EOF() || SB_GET() != TELQUAL_IS)
 		return;
+	sb_auth_complete();
 	settimer(xdisplocsubopt);
 	subpointer[SB_LEN()] = '\0';
 	(void)setenv("DISPLAY", (char *)subpointer, 1);
@@ -1267,6 +1278,7 @@ suboption()
 
 	if (SB_EOF())
 		return;
+	sb_auth_complete();
 	c = SB_GET();
 	if (c == TELQUAL_IS) {
 		if (subchar == TELOPT_OLD_ENVIRON)
@@ -1420,10 +1432,12 @@ suboption()
 		case NEW_ENV_VAR:
 		case ENV_USERVAR:
 			*cp = '\0';
-			if (valp)
-				(void)setenv(varp, valp, 1);
-			else
-				unsetenv(varp);
+			if (envvarok(varp)) {
+				if (valp)
+					(void)setenv(varp, valp, 1);
+				else
+					unsetenv(varp);
+			}
 			cp = varp = (char *)subpointer;
 			valp = 0;
 			break;
@@ -1439,10 +1453,12 @@ suboption()
 		}
 	}
 	*cp = '\0';
-	if (valp)
-		(void)setenv(varp, valp, 1);
-	else
-		unsetenv(varp);
+	if (envvarok(varp)) {
+		if (valp)
+			(void)setenv(varp, valp, 1);
+		else
+			unsetenv(varp);
+	}
 	break;
     }  /* end of case TELOPT_NEW_ENVIRON */
 #if	defined(AUTHENTICATION)
@@ -1458,15 +1474,17 @@ suboption()
 		 */
 		break;
 	case TELQUAL_IS:
-		auth_is(subpointer, SB_LEN());
+		if (!auth_negotiated)
+			auth_is(subpointer, SB_LEN());
 		break;
 	case TELQUAL_NAME:
-		auth_name(subpointer, SB_LEN());
+		if (!auth_negotiated)
+			auth_name(subpointer, SB_LEN());
 		break;
 	}
 	break;
 #endif
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
     case TELOPT_ENCRYPT:
 	if (SB_EOF())
 		break;
@@ -1508,7 +1526,7 @@ suboption()
 		break;
 	}
 	break;
-#endif
+#endif	/* ENCRYPTION */
 
     default:
 	break;
@@ -1522,8 +1540,8 @@ doclientstat()
 	clientstat(TELOPT_LINEMODE, WILL, 0);
 }
 
-#define	ADD(c)	 *ncp++ = c
-#define	ADD_DATA(c) { *ncp++ = c; if (c == SE || c == IAC) *ncp++ = c; }
+#define	ADD(c)	 *ncp++ = c;
+#define	ADD_DATA(c) { *ncp++ = c; if (c == SE) *ncp++ = c; }
 	void
 send_status()
 {
@@ -1552,10 +1570,14 @@ send_status()
 		if (my_want_state_is_will(i)) {
 			ADD(WILL);
 			ADD_DATA(i);
+			if (i == IAC)
+				ADD(IAC);
 		}
 		if (his_want_state_is_will(i)) {
 			ADD(DO);
 			ADD_DATA(i);
+			if (i == IAC)
+				ADD(IAC);
 		}
 	}
 
@@ -1570,14 +1592,15 @@ send_status()
 		ADD(SE);
 
 		if (restartany >= 0) {
-			ADD(SB);
+			ADD(SB)
 			ADD(TELOPT_LFLOW);
 			if (restartany) {
 				ADD(LFLOW_RESTART_ANY);
 			} else {
 				ADD(LFLOW_RESTART_XON);
 			}
-			ADD(SE);
+			ADD(SE)
+			ADD(SB);
 		}
 	}
 
@@ -1590,6 +1613,8 @@ send_status()
 		ADD(TELOPT_LINEMODE);
 		ADD(LM_MODE);
 		ADD_DATA(editmode);
+		if (editmode == IAC)
+			ADD(IAC);
 		ADD(SE);
 
 		ADD(SB);
@@ -1613,3 +1638,35 @@ send_status()
 	DIAG(TD_OPTIONS,
 		{printsub('>', statusbuf, ncp - statusbuf); netflush();});
 }
+
+static int envvarok(varp)
+	char *varp;
+{
+	if (!strchr(varp, '=') &&
+	    strcmp(varp, "TERMCAP") && /* to prevent a security hole  */
+	    strcmp(varp, "TERMINFO") &&	/* with tgetent */
+	    strcmp(varp, "TERMPATH") &&
+	    strcmp(varp, "HOME") && /* to prevent the tegetent bug  */
+	    strncmp(varp, "LD_", strlen("LD_")) && /* most systems */
+	    strncmp(varp, "_RLD_", strlen("_RLD_")) && /* irix */
+	    strncmp(varp, "KRB5", strlen("KRB5")) && /* v5 */
+	    /* The above is a catch-all for now.  Here are some of the
+	       specific ones we must avoid passing, at least until we
+	       can prove it can be done safely.  Keep this list around
+	       in case someone wants to remove the catch-all.  */
+	    strcmp(varp, "KRB5_CONFIG") && /* v5 */
+	    strcmp(varp, "KRB5CCNAME") &&  /* v5 */
+	    strcmp(varp, "KRB5_KTNAME") && /* v5 */
+	    strcmp(varp, "KRBTKFILE") &&   /* v4 */
+	    strcmp(varp, "KRB_CONF") &&	   /* cns v4 */
+	    strcmp(varp, "KRB_REALMS") &&  /* cns v4 */
+	    strcmp(varp, "LIBPATH") &&     /* AIX */
+	    strcmp(varp, "RESOLV_HOST_CONF") && /* linux */
+	    strcmp(varp, "IFS")) {
+		return 1;
+	} else {
+		syslog(LOG_INFO, "Rejected the attempt to modify the environment variable \"%s\"", varp);
+		return 0;
+	}
+
+}
