@@ -37,11 +37,7 @@ static char copyright[] =
 	The Regents of the University of California.  All rights reserved.\n";
 #endif /* not lint */
 
-#ifndef lint
-static char sccsid[] = "@(#)telnetd.c	8.4 (Berkeley) 5/30/95";
-#endif /* not lint */
-
-#include <config.h>
+/* based on @(#)telnetd.c	8.1 (Berkeley) 6/4/93 */
 
 #include "telnetd.h"
 #include "pathnames.h"
@@ -55,6 +51,11 @@ static char sccsid[] = "@(#)telnetd.c	8.4 (Berkeley) 5/30/95";
 # undef _SC_CRAY_SECURE_SYS
 #endif
 
+#include <stdio.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <libpty.h>
+#include <com_err.h>
 #if	defined(_SC_CRAY_SECURE_SYS)
 #include <sys/sysv.h>
 #include <sys/secdev.h>
@@ -73,27 +74,33 @@ struct	socket_security ss;
 # endif /* SO_SEC_MULTI */
 #endif	/* _SC_CRAY_SECURE_SYS */
 
+#ifdef KRB5
+#include "krb5.h"
+#endif
+
 #if	defined(AUTHENTICATION)
 #include <libtelnet/auth.h>
-int	auth_level = 0;
+#include <libtelnet/auth-proto.h>
 #endif
-#if	defined(SecurID)
-int	require_SecurID = 0;
+#ifdef ENCRYPTION
+#include <libtelnet/encrypt.h>
+#include <libtelnet/enc-proto.h>
 #endif
 
-extern	int utmp_len;
 int	registerd_host_only = 0;
 
 #ifdef	STREAMSPTY
+#include <sys/stream.h>
 # include <stropts.h>
 # include <termio.h>
 /* make sure we don't get the bsd version */
-# ifndef __sgi
+#ifdef HAVE_SYS_TTY_H
 # include "/usr/include/sys/tty.h"
+#endif
+#ifdef  HAS_PTYVAR
 # include <sys/ptyvar.h>
-# else  /* __sgi */
-# include <sys/strtty.h>
-# endif /* __sgi */
+#endif
+
 
 /*
  * Because of the way ptyibuf is used with streams messages, we need
@@ -120,6 +127,8 @@ char	ptyibuf2[BUFSIZ];
 
 #endif /* ! STREAMPTY */
 
+void doit P((struct sockaddr_in *));
+
 int	hostinfo = 1;			/* do we print login banner? */
 
 #ifdef	CRAY
@@ -131,6 +140,10 @@ int debug = 0;
 int keepalive = 1;
 char *progname;
 
+int maxhostlen = 0;
+int always_ip = 0;
+int stripdomain = 1;
+
 extern void usage P((void));
 
 /*
@@ -139,7 +152,8 @@ extern void usage P((void));
  * passed off to getopt().
  */
 char valid_opts[] = {
-	'd', ':', 'h', 'k', 'n', 'S', ':', 'u', ':', 'U',
+	'd', ':', 'h', 'k', 'L', ':', 'n', 'S', ':', 'U',
+	'w',
 #ifdef	AUTHENTICATION
 	'a', ':', 'X', ':',
 #endif
@@ -149,6 +163,9 @@ char valid_opts[] = {
 #ifdef DIAGNOSTICS
 	'D', ':',
 #endif
+#ifdef	ENCRYPTION
+	'e', ':',
+#endif
 #if	defined(CRAY) && defined(NEWINIT)
 	'I', ':',
 #endif
@@ -161,11 +178,14 @@ char valid_opts[] = {
 #ifdef	SecurID
 	's',
 #endif
-	'L', ':',
+#ifdef KRB5
+	'R', ':', 't', ':',
+#endif
 	'\0'
 };
 
 main(argc, argv)
+	int argc;
 	char *argv[];
 {
 	struct sockaddr_in from;
@@ -180,11 +200,11 @@ main(argc, argv)
 	pfrontp = pbackp = ptyobuf;
 	netip = netibuf;
 	nfrontp = nbackp = netobuf;
+#ifdef	ENCRYPTION
+	nclearto = 0;
+#endif	/* ENCRYPTION */
 
 	progname = *argv;
-#if	defined(ENCRYPTION)
-	nclearto = 0;
-#endif
 
 #ifdef CRAY
 	/*
@@ -262,6 +282,17 @@ main(argc, argv)
 			break;
 #endif /* DIAGNOSTICS */
 
+#ifdef	ENCRYPTION
+		case 'e':
+			if (strcmp(optarg, "debug") == 0) {
+				extern int encrypt_debug_mode;
+				encrypt_debug_mode = 1;
+				break;
+			}
+			usage();
+			/* NOTREACHED */
+			break;
+#endif	/* ENCRYPTION */
 
 		case 'h':
 			hostinfo = 0;
@@ -290,6 +321,14 @@ main(argc, argv)
 #endif	/* defined(LINEMODE) && defined(KLUDGELINEMODE) */
 			break;
 
+		case 'L':
+		    {
+		        extern char *login_program;
+
+			login_program = optarg;
+			break;
+		    }
+
 		case 'n':
 			keepalive = 0;
 			break;
@@ -322,6 +361,25 @@ main(argc, argv)
 		    }
 #endif	/* CRAY */
 
+#ifdef KRB5
+		case 'R':
+		    {
+			extern krb5_context telnet_context;
+			krb5_error_code retval;
+			
+			if (telnet_context == 0) {
+				retval = krb5_init_context(&telnet_context);
+				if (retval) {
+					com_err("telnetd", retval,
+						"while initializing krb5");
+					exit(1);
+				}
+			}
+			krb5_set_default_realm(telnet_context, optarg);
+			break;
+		    }
+#endif	/* KRB5 */
+
 #ifdef	SecurID
 		case 's':
 			/* SecurID required */
@@ -340,9 +398,16 @@ main(argc, argv)
 #endif
 			break;
 
-		case 'u':
-			utmp_len = atoi(optarg);
+#ifdef KRB5
+		case 't':
+		    {
+			extern char *telnet_srvtab;
+
+			telnet_srvtab = optarg;
 			break;
+		    }
+#endif	/* KRB5 */
+
 
 		case 'U':
 			registerd_host_only = 1;
@@ -356,11 +421,37 @@ main(argc, argv)
 			auth_disable_name(optarg);
 			break;
 #endif	/* AUTHENTICATION */
-
-		case 'L':
-			new_login = optarg;
+		case 'w':
+			if (!strcmp(optarg, "ip"))
+				always_ip = 1;
+			else {
+				char *cp;
+				cp = strchr(optarg, ',');
+				if (cp == NULL)
+					maxhostlen = atoi(optarg);
+				else if (*(++cp)) {
+					if (!strcmp(cp, "striplocal"))
+						stripdomain = 1;
+					else if (!strcmp(cp, "nostriplocal"))
+						stripdomain = 0;
+					else {
+						usage();
+					}
+					*(--cp) = '\0';
+					maxhostlen = atoi(optarg);
+				}
+			}
 			break;
-			
+		case 'u':
+			maxhostlen = atoi(optarg);
+			break;
+		case 'i':
+			always_ip = 1;
+			break;
+		case 'N':
+			stripdomain = 0;
+			break;
+
 		default:
 			fprintf(stderr, "telnetd: %c: unknown option\n", ch);
 			/* FALLTHROUGH */
@@ -382,7 +473,7 @@ main(argc, argv)
 		usage();
 		/* NOT REACHED */
 	    } else if (argc == 1) {
-		    if (sp = getservbyname(*argv, "tcp")) {
+		    if ((sp = getservbyname(*argv, "tcp"))) {
 			sin.sin_port = sp->s_port;
 		    } else {
 			sin.sin_port = atoi(*argv);
@@ -523,7 +614,9 @@ main(argc, argv)
 #endif	/* defined(IPPROTO_IP) && defined(IP_TOS) */
 	net = 0;
 	doit(&from);
+	
 	/* NOTREACHED */
+	return 0;
 }  /* end of main */
 
 	void
@@ -567,11 +660,22 @@ usage()
 #ifdef	AUTHENTICATION
 	fprintf(stderr, " [-X auth-type]");
 #endif
-	fprintf(stderr, " [-u utmp_hostname_length] [-U]");
+	fprintf(stderr, " [-u utmp_hostname_length] [-U]\n");
+	fprintf(stderr, " [-w [ip|maxhostlen[,[no]striplocal]]]\n");
 	fprintf(stderr, " [port]\n");
 	exit(1);
 }
 
+static void encrypt_failure()
+{
+    char *error_message =
+	"Encryption was not successfully negotiated.  Goodbye.\r\n\r\n";
+
+    writenet(error_message, strlen(error_message));
+    netflush();
+    exit(1);
+}
+
 /*
  * getterminaltype
  *
@@ -602,19 +706,20 @@ getterminaltype(name)
     }
 #endif
 
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
     send_will(TELOPT_ENCRYPT, 1);
-    send_do(TELOPT_ENCRYPT, 1);	/* esc@magic.fi */
-#endif
+    send_do(TELOPT_ENCRYPT, 1);
+#endif	/* ENCRYPTION */
     send_do(TELOPT_TTYPE, 1);
     send_do(TELOPT_TSPEED, 1);
     send_do(TELOPT_XDISPLOC, 1);
     send_do(TELOPT_NEW_ENVIRON, 1);
     send_do(TELOPT_OLD_ENVIRON, 1);
     while (
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 	   his_do_dont_is_changing(TELOPT_ENCRYPT) ||
-#endif
+	   his_will_wont_is_changing(TELOPT_ENCRYPT) ||
+#endif	/* ENCRYPTION */
 	   his_will_wont_is_changing(TELOPT_TTYPE) ||
 	   his_will_wont_is_changing(TELOPT_TSPEED) ||
 	   his_will_wont_is_changing(TELOPT_XDISPLOC) ||
@@ -622,7 +727,7 @@ getterminaltype(name)
 	   his_will_wont_is_changing(TELOPT_OLD_ENVIRON)) {
 	ttloop();
     }
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
     /*
      * Wait for the negotiation of what type of encryption we can
      * send with.  If autoencrypt is not set, this will just return.
@@ -630,45 +735,58 @@ getterminaltype(name)
     if (his_state_is_will(TELOPT_ENCRYPT)) {
 	encrypt_wait();
     }
-#endif
+    if (auth_must_encrypt()) {
+	time_t timeout = time(0) + 60;
+	
+	if (my_state_is_dont(TELOPT_ENCRYPT) ||
+	    my_state_is_wont(TELOPT_ENCRYPT))
+	    encrypt_failure();
+
+	if (!EncryptStartInput() || !EncryptStartOutput())
+	    encrypt_failure();
+
+	while (!encrypt_is_encrypting()) {
+	    if (time(0) > timeout)
+		encrypt_failure();
+	    ttloop();
+	}
+    }
+#endif	/* ENCRYPTION */
+    /* Options like environment require authentication and encryption
+       negotiation to be completed.*/
+    auth_negotiated = 1;
     if (his_state_is_will(TELOPT_TSPEED)) {
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_TSPEED, TELQUAL_SEND, IAC, SE };
 
-	memmove(nfrontp, sb, sizeof sb);
+	memcpy(nfrontp, sb, sizeof sb);
 	nfrontp += sizeof sb;
-	DIAG(TD_OPTIONS, printsub('>', sb + 2, sizeof sb - 2););
     }
     if (his_state_is_will(TELOPT_XDISPLOC)) {
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_XDISPLOC, TELQUAL_SEND, IAC, SE };
 
-	memmove(nfrontp, sb, sizeof sb);
+	memcpy(nfrontp, sb, sizeof sb);
 	nfrontp += sizeof sb;
-	DIAG(TD_OPTIONS, printsub('>', sb + 2, sizeof sb - 2););
     }
     if (his_state_is_will(TELOPT_NEW_ENVIRON)) {
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_NEW_ENVIRON, TELQUAL_SEND, IAC, SE };
 
-	memmove(nfrontp, sb, sizeof sb);
+	memcpy(nfrontp, sb, sizeof sb);
 	nfrontp += sizeof sb;
-	DIAG(TD_OPTIONS, printsub('>', sb + 2, sizeof sb - 2););
     }
     else if (his_state_is_will(TELOPT_OLD_ENVIRON)) {
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_OLD_ENVIRON, TELQUAL_SEND, IAC, SE };
 
-	memmove(nfrontp, sb, sizeof sb);
+	memcpy(nfrontp, sb, sizeof sb);
 	nfrontp += sizeof sb;
-	DIAG(TD_OPTIONS, printsub('>', sb + 2, sizeof sb - 2););
     }
     if (his_state_is_will(TELOPT_TTYPE)) {
 
-	memmove(nfrontp, ttytype_sbbuf, sizeof ttytype_sbbuf);
+	memcpy(nfrontp, ttytype_sbbuf, sizeof ttytype_sbbuf);
 	nfrontp += sizeof ttytype_sbbuf;
-	DIAG(TD_OPTIONS, printsub('>', ttytype_sbbuf + 2,
-					sizeof ttytype_sbbuf - 2););
     }
     if (his_state_is_will(TELOPT_TSPEED)) {
 	while (sequenceIs(tspeedsubopt, baseline))
@@ -697,12 +815,14 @@ getterminaltype(name)
 	 * we have to just go with what we (might) have already gotten.
 	 */
 	if (his_state_is_will(TELOPT_TTYPE) && !terminaltypeok(terminaltype)) {
-	    (void) strncpy(first, terminaltype, sizeof(first));
+	    (void) strncpy(first, terminaltype, sizeof(first) - 1);
+	    first[sizeof(first) - 1] = '\0';
 	    for(;;) {
 		/*
 		 * Save the unknown name, and request the next name.
 		 */
-		(void) strncpy(last, terminaltype, sizeof(last));
+		(void) strncpy(last, terminaltype, sizeof(last) - 1);
+		last[sizeof(last) - 1] = '\0';
 		_gettermname();
 		if (terminaltypeok(terminaltype))
 		    break;
@@ -719,9 +839,12 @@ getterminaltype(name)
 		     * RFC1091 compliant telnets will cycle back to
 		     * the start of the list.
 		     */
-		     _gettermname();
-		    if (strncmp(first, terminaltype, sizeof(first)) != 0)
-			(void) strncpy(terminaltype, first, sizeof(first));
+		    _gettermname();
+		    if (strncmp(first, terminaltype, sizeof(first)) != 0) {
+			(void) strncpy(terminaltype, first,
+				       sizeof(terminaltype) - 1);
+			terminaltype[sizeof(terminaltype) - 1] = '\0';
+		    }
 		    break;
 		}
 	    }
@@ -741,10 +864,8 @@ _gettermname()
     if (his_state_is_wont(TELOPT_TTYPE))
 	return;
     settimer(baseline);
-    memmove(nfrontp, ttytype_sbbuf, sizeof ttytype_sbbuf);
+    memcpy(nfrontp, ttytype_sbbuf, sizeof ttytype_sbbuf);
     nfrontp += sizeof ttytype_sbbuf;
-    DIAG(TD_OPTIONS, printsub('>', ttytype_sbbuf + 2,
-					sizeof ttytype_sbbuf - 2););
     while (sequenceIs(ttypesubopt, baseline))
 	ttloop();
 }
@@ -755,7 +876,7 @@ terminaltypeok(s)
 {
     char buf[1024];
 
-    if (terminaltype == NULL)
+    if (!*s)
 	return(1);
 
     /*
@@ -769,14 +890,18 @@ terminaltypeok(s)
 	return(0);
     return(1);
 }
+#if HAVE_ARPA_NAMESER_H
+#include <arpa/nameser.h>
+#endif
 
-#ifndef	MAXHOSTNAMELEN
-#define	MAXHOSTNAMELEN 64
-#endif	/* MAXHOSTNAMELEN */
+#ifndef MAXDNAME
+#define MAXDNAME 256 /*per the rfc*/
+#endif
 
 char *hostname;
-char host_name[MAXHOSTNAMELEN];
-char remote_host_name[MAXHOSTNAMELEN];
+char host_name[MAXDNAME];
+char remote_host_name[MAXDNAME];
+char *rhost_sane;
 
 #ifndef	convex
 extern void telnet P((int, int));
@@ -787,46 +912,32 @@ extern void telnet P((int, int, char *));
 /*
  * Get a pty, scan input lines.
  */
-doit(who)
+void doit(who)
 	struct sockaddr_in *who;
 {
 	char *host, *inet_ntoa();
-	int t;
 	struct hostent *hp;
 	int level;
 	int ptynum;
 	char user_name[256];
-
+long retval;
 	/*
 	 * Find an available pty to use.
 	 */
-#ifndef	convex
-	pty = getpty(&ptynum);
-	if (pty < 0)
-		fatal(net, "All network ports in use");
-#else
-	for (;;) {
-		char *lp;
-		extern char *line, *getpty();
+pty_init();
+	
 
-		if ((lp = getpty()) == NULL)
-			fatal(net, "Out of ptys");
-
-		if ((pty = open(lp, 2)) >= 0) {
-			strcpy(line,lp);
-			line[5] = 't';
-			break;
-		}
+	if ((retval = pty_getpty(&pty, line, 17)) != 0) {
+		fatal(net, error_message(retval));
 	}
-#endif
 
 #if	defined(_SC_CRAY_SECURE_SYS)
 	/*
-	 *	set ttyp line security label
+	 *	set ttyp line security label 
 	 */
 	if (secflag) {
 		char slave_dev[16];
-
+/*xxx This code needs to be fixed to work without ptynum; I don't understand why they don't currently use line, so I don't really know how to fix.*/
 		sprintf(tty_dev, "/dev/pty/%03d", ptynum);
 		if (setdevs(tty_dev, &dv) < 0)
 		 	fatal(net, "cannot set pty security");
@@ -836,67 +947,50 @@ doit(who)
 	}
 #endif	/* _SC_CRAY_SECURE_SYS */
 
+	retval = pty_make_sane_hostname(who, maxhostlen,
+					stripdomain, always_ip,
+					&rhost_sane);
+	if (retval) {
+		fatal(net, error_message(retval));
+	}
 	/* get name of connected client */
 	hp = gethostbyaddr((char *)&who->sin_addr, sizeof (struct in_addr),
 		who->sin_family);
 
 	if (hp == NULL && registerd_host_only) {
 		fatal(net, "Couldn't resolve your address into a host name.\r\n\
-	 Please contact your net administrator");
-	} else if (hp) {
-		host = hp->h_name;
-	} else {
-		host = inet_ntoa(who->sin_addr);
+         Please contact your net administrator");
 	}
-	/*
-	 * We must make a copy because Kerberos is probably going
-	 * to also do a gethost* and overwrite the static data...
-	 */
-	strncpy(remote_host_name, host, sizeof(remote_host_name)-1);
-	remote_host_name[sizeof(remote_host_name)-1] = 0;
-	host = remote_host_name;
 
 	(void) gethostname(host_name, sizeof (host_name));
 	hostname = host_name;
 
-#define abs(x) ((x < 0) ? (-x) : x)
-
-#define TRIM_HOSTNAME
-#ifdef  TRIM_HOSTNAME
-      /* Only trim if too long (and possible) */
-      if (strlen(remote_host_name) > abs(utmp_len)) {
-              char *domain = strchr(host_name, '.');
-              char *p = strchr(remote_host_name, '.');
-              if (domain && p && (strcmp(p, domain) == 0))
-                      *p = 0; /* remove domain part */
-      }
-#endif /* TRIM_HOSTNAME */
-
-      /*
-       * If hostname still doesn't fit utmp, use ipaddr.
-       */
-      if (strlen(remote_host_name) > abs(utmp_len))
-              strncpy(remote_host_name,
-                      inet_ntoa(who->sin_addr),
-                      sizeof(remote_host_name)-1);
-
 #if	defined(AUTHENTICATION) || defined(ENCRYPTION)
-	auth_encrypt_init(hostname, host, "TELNETD", 1);
+	auth_encrypt_init(hostname, rhost_sane, "TELNETD", 1);
 #endif
 
 	init_env();
+
+#ifdef	SIGTTOU
+	/*
+	 * Ignoring SIGTTOU keeps the kernel from blocking us.
+	 * we tweak the tty with an ioctl()
+	 * (in ttioct() in /sys/tty.c in a BSD kernel)
+	 */
+	(void) signal(SIGTTOU, SIG_IGN);
+#endif
 	/*
 	 * get terminal type.
 	 */
 	*user_name = 0;
 	level = getterminaltype(user_name);
-	setenv("TERM", terminaltype ? terminaltype : "network", 1);
+	setenv("TERM", *terminaltype ? terminaltype : "network", 1);
 
 	/*
 	 * Start up the login process on the slave side of the terminal
 	 */
 #ifndef	convex
-	startslave(host, level, user_name);
+	startslave(rhost_sane, level, user_name);
 
 #if	defined(_SC_CRAY_SECURE_SYS)
 	if (secflag) {
@@ -909,7 +1003,7 @@ doit(who)
 
 	telnet(net, pty);  /* begin server processing */
 #else
-	telnet(net, pty, host);
+	telnet(net, pty, rhost_sane);
 #endif
 	/*NOTREACHED*/
 }  /* end of doit */
@@ -949,11 +1043,10 @@ telnet(f, p, host)
 	char	defent[TABBUFSIZ];
 	char	defstrs[TABBUFSIZ];
 #undef	TABBUFSIZ
-	char *HE;
+	char *HEstr;
 	char *HN;
 	char *IM;
 	void netflush();
-	int nfd;
 
 	/*
 	 * Initialize the slc mapping table.
@@ -1090,13 +1183,6 @@ telnet(f, p, host)
 #ifdef	SIGTSTP
 	(void) signal(SIGTSTP, SIG_IGN);
 #endif
-#ifdef	SIGTTOU
-	/*
-	 * Ignoring SIGTTOU keeps the kernel from blocking us
-	 * in ttioct() in /sys/tty.c.
-	 */
-	(void) signal(SIGTTOU, SIG_IGN);
-#endif
 
 	(void) signal(SIGCHLD, cleanup);
 
@@ -1148,7 +1234,7 @@ telnet(f, p, host)
 		char *getstr();
 		char *cp=defstrs;
 
-		HE = getstr("he", &cp);
+		HEstr = getstr("he", &cp);
 		HN = getstr("hn", &cp);
 		IM = getstr("im", &cp);
 		if (HN && *HN)
@@ -1157,9 +1243,9 @@ telnet(f, p, host)
 			IM = "";
 	} else {
 		IM = DEFAULT_IM;
-		HE = 0;
+		HEstr = 0;
 	}
-	edithost(HE, host_name);
+	edithost(HEstr, host_name);
 	if (hostinfo && *IM)
 		putf(IM, ptyibuf2);
 
@@ -1183,7 +1269,6 @@ telnet(f, p, host)
 	startslave(host);
 #endif
 
-	nfd = ((f > p) ? f : p) + 1;
 	for (;;) {
 		fd_set ibits, obits, xbits;
 		register int c;
@@ -1215,7 +1300,7 @@ telnet(f, p, host)
 		if (!SYNCHing) {
 			FD_SET(f, &xbits);
 		}
-		if ((c = select(nfd, &ibits, &obits, &xbits,
+		if ((c = select(16, &ibits, &obits, &xbits,
 						(struct timeval *)0)) < 1) {
 			if (c == -1) {
 				if (errno == EINTR) {
@@ -1354,9 +1439,6 @@ telnet(f, p, host)
 					*nfrontp++ = IAC;
 					*nfrontp++ = DM;
 					neturg = nfrontp-1; /* off by one XXX */
-					DIAG(TD_OPTIONS,
-					    printoption("td: send IAC", DM));
-
 #endif
 				}
 				if (his_state_is_will(TELOPT_LFLOW) &&
@@ -1373,9 +1455,6 @@ telnet(f, p, host)
 								 : LFLOW_OFF,
 							IAC, SE);
 						nfrontp += 6;
-						DIAG(TD_OPTIONS, printsub('>',
-						    (unsigned char *)nfrontp-4,
-						    4););
 					}
 				}
 				pcc--;
@@ -1434,7 +1513,7 @@ telnet(f, p, host)
 	}
 	cleanup(0);
 }  /* end of telnet */
-
+	
 #ifndef	TCSIG
 # ifdef	TIOCSIG
 #  define TCSIG TIOCSIG
@@ -1493,28 +1572,13 @@ int readstream(p, ibuf, bufsize)
 
 	case M_IOCTL:
 		ip = (struct iocblk *) (ibuf+1);
-
-		switch (ip->ioc_cmd) {
-		case TCSETS:
-		case TCSETSW:
-		case TCSETSF:
-			tsp = (struct termios *)
-					(ibuf+1 + sizeof(struct iocblk));
-			vstop = tsp->c_cc[VSTOP];
-			vstart = tsp->c_cc[VSTART];
-			ixon = tsp->c_iflag & IXON;
-			break;
-		case TCSETA:
-		case TCSETAW:
-		case TCSETAF:
-			tp = (struct termio *) (ibuf+1 + sizeof(struct iocblk));
-			vstop = tp->c_cc[VSTOP];
-			vstart = tp->c_cc[VSTART];
-			ixon = tp->c_iflag & IXON;
-			break;
-		default:
-			errno = EAGAIN;
-			return(-1);
+		if (readstream_termio(ip->ioc_cmd, ibuf, 
+				      &vstop, &vstart, &ixon)) {
+		  if (readstream_termios(ip->ioc_cmd, ibuf, 
+					 &vstop, &vstart, &ixon)) {
+		    errno = EAGAIN;
+		    return(-1);
+		  }
 		}
 
 		newflow =  (ixon && (vstart == 021) && (vstop == 023)) ? 1 : 0;
@@ -1541,14 +1605,6 @@ interrupt()
 {
 	ptyflush();	/* half-hearted */
 
-#if defined(STREAMSPTY) && defined(TIOCSIGNAL)
-	/* Streams PTY style ioctl to post a signal */
-	{
-		int sig = SIGINT;
-		(void) ioctl(pty, TIOCSIGNAL, &sig);
-		(void) ioctl(pty, I_FLUSH, FLUSHR);
-	}
-#else
 #ifdef	TCSIG
 	(void) ioctl(pty, TCSIG, (char *)SIGINT);
 #else	/* TCSIG */
@@ -1556,7 +1612,6 @@ interrupt()
 	*pfrontp++ = slctab[SLC_IP].sptr ?
 			(unsigned char)*slctab[SLC_IP].sptr : '\177';
 #endif	/* TCSIG */
-#endif
 }
 
 /*
@@ -1623,27 +1678,3 @@ doeof()
 	*pfrontp++ = slctab[SLC_EOF].sptr ?
 			(unsigned char)*slctab[SLC_EOF].sptr : '\004';
 }
-
-/* sigh -- make libtermcap.a happy */
-#ifdef SOLARIS2
-char *index(char *a, int b)
-{
-	return(strchr(a,b));
-}
-char *rindex(char *a, int b)
-{
-	return(strrchr(a,b));
-}
-void bzero(void *sp, int len)
-{
-	memset(sp, 0, len);
-}
-int bcopy(void *s1, void *s2, int len)
-{
-	memcpy(s2, s1, len);
-	return 0;
-}
-int bcmp(void *s1, void *s2, int len) {
-	memcmp(s2, s1, len);
-}
-#endif
