@@ -31,11 +31,7 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static char sccsid[] = "@(#)termstat.c	8.2 (Berkeley) 5/30/95";
-#endif /* not lint */
-
-#include <config.h>
+/* based on @(#)termstat.c	8.1 (Berkeley) 6/4/93 */
 
 #include "telnetd.h"
 
@@ -183,7 +179,7 @@ localstat()
 		tty_setlinemode(uselinemode);
 	}
 
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 	/*
 	 * If the terminal is not echoing, but editing is enabled,
 	 * something like password input is going to happen, so
@@ -201,8 +197,8 @@ localstat()
 			enc_passwd = 0;
 		}
 	}
-#endif
-	
+#endif	/* ENCRYPTION */
+
 	/*
 	 * Do echo mode handling as soon as we know what the
 	 * linemode is going to be.
@@ -322,7 +318,7 @@ localstat()
 			nfrontp += 7;
 			editmode = useeditmode;
 		}
-
+							
 
 		/*
 		 * Check for changes to special characters in use.
@@ -424,7 +420,7 @@ clientstat(code, parm1, parm2)
 					uselinemode = 1;
 				}
 			}
-
+		
 			/*
 			 * Quit now if we can't do it.
 			 */
@@ -465,7 +461,7 @@ clientstat(code, parm1, parm2)
 				send_will(TELOPT_ECHO, 1);
 		}
 		break;
-
+	
 	case LM_MODE:
 	    {
 		register int ack, changed;
@@ -514,7 +510,7 @@ clientstat(code, parm1, parm2)
  					IAC, SE);
  				nfrontp += 7;
  			}
-
+ 		
 			editmode = useeditmode;
 		}
 
@@ -548,9 +544,9 @@ clientstat(code, parm1, parm2)
 		(void) ioctl(pty, TIOCSWINSZ, (char *)&ws);
 	    }
 #endif	/* TIOCSWINSZ */
-
+		
 		break;
-
+	
 	case TELOPT_TSPEED:
 	    {
 		def_tspeed = parm1;
@@ -615,7 +611,7 @@ _termstat()
  *
  * Some things should not be done until after the login process has started
  * and all the pty modes are set to what they are supposed to be.  This
- * function is called when the pty state has been processed for the first time.
+ * function is called when the pty state has been processed for the first time. 
  * It calls other functions that do things that were deferred in each module.
  */
 	void
